#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""redis数据访问类"""

import redis
from .Env import Env


class Redis():
    _instance = dict()

    def __init__(self, host=None, port=6379, password=None, db=0):
        '''
        构造方法
        :param host: host
        :param port: port
        :param password: 密码
        :param db: 默认db
        '''
        env = Env()
        if host is not None:
            self.host = host
            self.port = port
            self.password = password
            self.db = db
        else:
            self.host = env("REDIS_HOST", 'localhost')
            self.port = int(env("REDIS_PORT", 6379))
            self.password = env('REDIS_PASSWORD', None)
            self.db = int(env("REDIS_DB", 0))

        if self._instance.get(self.db) is None:
            pool = redis.ConnectionPool(host=self.host, port=self.port, db=self.db, password=self.password, decode_responses=True)
            self._instance[self.db] = redis.Redis(connection_pool=pool)

    def select(self, db):
        '''
        切换redis库
        :param db: 库index
        :return: redis.Redis
        '''
        if type(db) != int:
            raise Exception('select db must be the type of int')
        if self._instance.get(db) is None:
            pool = redis.ConnectionPool(host=self.host, port=self.port, db=db, password=self.password, decode_responses=True)
            self._instance[db] = redis.Redis(connection_pool=pool)
        return self._instance[db]

    def __getattr__(self, key):
        '''
        默认库则调用
        :param key: redis 执行方法
        :return: result
        '''
        return self._instance.get(self.db).__getattribute__(key)




from .Env import Env
import oss2
from oss2.models import BucketCors, CorsRule, BucketReferer


class Oss():
    '''
    阿里OSS服务类
    '''
    _oss_sinstance = None

    def __init__(self, access_key=None, access_secret=None, end_point=None, bucket_name=None):
        if access_key is not None:
            self._oss_sinstance = oss2.Bucket(
                oss2.Auth(access_key, access_secret), end_point, bucket_name
            )
        else:
            env = Env()
            self._oss_sinstance = oss2.Bucket(
                oss2.Auth(env('OSS_ACCESS_KEY'), env('OSS_ACCESS_SECRET')),
                env('OSS_END_POINT'), env('OSS_BUCKET_NAME'),
            )

    def push(self, filename, content, header=None):
        '''
        上传文件
        :param filename: 上传文件名， 例如：'data/test.json'
        :param content: 上传的文件内容
        :param header: header
        :return:
        '''
        return self._oss_sinstance.put_object(filename, content, headers=header)

    def push_file(self, filename, local_filename, header=None):
        '''
        上传本地文件
        :param filename: 上传文件名， 例如：'data/test.json'
        :param local_filename: '本地文件路径'
        :param header: header
        :return:
        '''
        return self._oss_sinstance.put_object_from_file(filename, local_filename, headers=header)

    def rule(self, allowed_origins=['*'], allowed_methods=['GET', 'POST', 'HEAD'], allowed_headers=['*'],
             max_age_seconds=600):
        '''
        处理跨域
        :param allowed_origins: 来源
        :param allowed_methods: 接受的请求方法
        :param allowed_headers: 接受的请求头
        :param max_age_seconds: 缓存时间（秒）
        :return:
        '''
        rule = CorsRule(allowed_origins=allowed_origins, allowed_methods=allowed_methods,
                        allowed_headers=allowed_headers, max_age_seconds=max_age_seconds)
        self._oss_sinstance.put_bucket_cors(BucketCors([rule]))
        return self

    def referer(self, referers=None):
        '''
        防盗链
        :param referers: ['http://www.longhash.com', 'http://api.longhash.com']
        :return: self
        '''
        if referers is not None:
            self._oss_sinstance.put_bucket_referer(BucketReferer(False, referers))
        else:
            self._oss_sinstance.put_bucket_referer(BucketReferer(True, []))
        return self



import asyncio


class Async():
    __slots__ = []

    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(Async, cls).__new__(cls, *args, **kwargs)
            return cls._instance

    def run(self, tasks):
        flag = asyncio._get_running_loop()
        if flag:
            loop = asyncio.get_event_loop()
            for index in tasks:
                asyncio.run_coroutine_threadsafe(index, loop=loop)
        else:
            loop = asyncio.new_event_loop()
            loop.run_until_complete(asyncio.wait(tasks))
            loop.close()


if __name__ == '__main__':
    async def hello2(index):
        print("number:{}, Hello world!".format(index))
        r = await asyncio.sleep(1)
        print("end!")


    tasks = [hello2(index) for index in range(5)]
    Async.run(tasks)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Log日志类"""

__author__ = ''

import logging
import datetime
from .Env import Env
from pathlib import Path


class Log(object):
    _instance = {}  # 安日志存储logging 实例对象

    def debug(self, msg):
        self.logger().debug(self._format_msg(msg))

    def info(self, msg):
        self.logger().info(self._format_msg(msg))

    def warning(self, msg):
        self.logger().warning(self._format_msg(msg))

    def error(self, msg):
        self.logger().error(self._format_msg(msg))

    def critical(self, msg):
        self.logger().critical(self._format_msg(msg))

    def logger(self):
        date = datetime.datetime.now().strftime("%Y%m%d")
        if self._instance.get(date) is None:
            logger = logging.getLogger("logging")
            formatter = logging.Formatter('%(asctime)s 【%(levelname)s】%(message)s')
            logger.setLevel(Env()('LOG_LEVEL', 'DEBUG'))  # log level
            fh = logging.FileHandler(self._logfile())
            fh.setFormatter(formatter)
            if Env()('LOG_DEBUG', 'false').lower() == 'true':  # debug
                sh = logging.StreamHandler()
                sh.setFormatter(formatter)
                logger.addHandler(sh)
            logger.addHandler(fh)
            self._instance[date] = logger
        return self._instance.get(date)

    def _logfile(self):
        '''
        处理日志文件路径
        :return:
        '''
        date = datetime.datetime.now().strftime("%Y%m%d")
        path = Env()('LOG_PATH', None)
        if path is not None and path:
            path = Path(path)
            if path.is_dir():
                return path.joinpath('{}.log'.format(date))
            return path
        return Path(Env.basedir).joinpath('Storage/Logs/{}.log'.format(date))  # 默认 ./Storage/Logs/**.log

    def _format_msg(self, msg):
        if isinstance(msg, BaseException):
            import traceback
            return traceback.format_exc()
        return msg.__str__()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
import pymysql
import codecs, os


class Artisan():
    author = 'jemes'                                            # 作者

    profix = 'lh_'                                              # 表前缀

    profix_filter = True                                        # 表前缀过滤 符合才生成model

    namespace = 'Models.Longhash'                               # 命名空间

    database = 'longhash'                                       # 数据库名称

    path = os.path.abspath(os.path.dirname(__file__))           # 当前路径

    output = None                                               # 输出路径

    config = {                                                  # 数据库配置
        'host': '127.0.0.1',
        'port': 3306,
        'user': 'dev',
        'password': 'dev',
        'db': 'database',
        'charset': 'utf8mb4',
    }

    def handle(self, args=sys.argv):
        '''
        处理model入口方法
        :param args: 命令行输入参数
        :return:
        '''
        self.check(args)
        if args.__len__() == 2:                                 # 只有一个表名称
            modelname = self.auth_model_name(args[1])
        else:
            modelname = self.define_model_name(args[2])
        tablename = args[1]
        if tablename[:self.profix.__len__()] != self.profix and self.profix_filter:  # 表面前缀不一致 则不处理
            return
        sql = self.table_sql(tablename)
        self.model(modelname, tablename, sql)

    def all(self):
        '''
        处理model入口方法
        :param args: 命令行输入参数
        :return:
        '''
        tables = [index[0] for index in self.tables()]
        for index in tables:
            self.handle([None, index])
        self.basemodel()

    def check(self, args):
        '''
        校验配置是否正确
        :param args:
        :return:
        '''
        if args.__len__() == 1:
            raise Exception('invalid params')
        if self.config is None:
            raise Exception('databse not config')
        if self.output is None:
            raise Exception('output path invalid')

    def auth_model_name(self, tablename):
        '''
        根据表名自动生成model名称
        :param tablename: 表名称
        :return:
        '''
        if tablename[:self.profix.__len__()] != self.profix:  # 表面前缀不一致 则不处理
            tablename = tablename.replace('-', '_').split('_')
        else:
            tablename = tablename[self.profix.__len__():].replace('-', '_').split('_')
        return ''.join([index.capitalize() for index in tablename]) + 'Model'

    def define_model_name(self, name):
        '''
        生成用户自定义model名称
        :param name: model名称
        :return:
        '''
        return name.capitalize() if name[-5:] == 'Model' else name.capitalize() + 'Model'

    def tables(self):
        '''
        显示sql
        :param tablesname:
        :return:
        '''
        db = pymysql.connect(**self.config)
        cursor = db.cursor()
        cursor.execute("show tables")
        result = cursor.fetchall()
        db.close()
        return result

    def table_sql(self, tablesname):
        '''
        显示sql
        :param tablesname:
        :return:
        '''
        db = pymysql.connect(**self.config)
        cursor = db.cursor()
        cursor.execute("show full columns from `%s`" % tablesname)
        result = cursor.fetchall()
        db.close()
        return result

    def model(self, modelname, tablename, sql):
        '''
        生成model
        :param name: 生成的model名称
        :param tablename: 表名称
        :param sql: 表的结构
        :return:
        '''
        columns = [index[0] for index in sql]
        with codecs.open(self.path + '/model.txt', "rb", "UTF-8") as f:
            s = f.read()
        _template_parameter = {
            'author': self.author,
            'namespace': self.namespace,
            'classname': modelname,
            'database': self.database,
            'tablename': tablename,
            'columns': self.format_columns(sql),
            'create_time': '\'create_time\'' if 'create_time' in columns else None,
            'update_time': '\'update_time\'' if 'update_time' in columns else None
        }
        s = s % _template_parameter
        with codecs.open(self.output + '/' + modelname + '.py', "wb", "UTF-8") as f:
            f.write(s)
            f.flush()

    def format_columns(self, sql):
        '''
        生成column & 注释
        :param sql:
        :return:
        '''
        columns = [index[0] for index in sql]
        max_value = max([index.__len__() for index in columns])
        comments = [index[-1] for index in sql]
        data = ['\'{}\', {} # {}'.format(columns[i], ' ' * (50 + max_value - columns[i].__len__()), comments[i]) for i in range(columns.__len__())]
        return '\n        '.join(data)

    def basemodel(self):
        '''
        生成base model.txt
        :return:
        '''
        with codecs.open(self.path + '/basemodel.txt', "rb", "UTF-8") as f:
            s = f.read()
        with codecs.open(self.output + '/BaseModel.py', "wb", "UTF-8") as f:
            f.write(s)
            f.flush()


if __name__ == '__main__':
    sql = Artisan().handle([None, 'lh_test'])
    # sql = Artisan().all()
    # sql = Artisan().basemodel()
    pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""容器类"""

from inspect import isclass, isfunction


class App(object):
    _shares = {}  # 共享实例
    _alias = {  # 别名
        'redis': 'simutil.Redis.Redis',
        'log': 'simutil.Log.Log',
        'request': 'simutil.Request.Request',
        'env': 'simutil.Env.Env',
        'async': 'simutil.Async.Async',
        'config': 'simutil.ConfigParser.ConfigParser',
        'oss': 'simutil.Oss.Oss',
    }

    _share_instance = None

    # Env为单例模式
    def __new__(cls, *args, **kw):
        if not cls._share_instance:
            cls._share_instance = super(App, cls).__new__(cls, *args, **kw)
        return cls._share_instance

    def register(self, key, alias):
        '''
        注册方法
        :param key: 注册类是对应的key
        :param alias: 注册的字符转，实例或者类
        :return: None
        '''
        if self._shares.get(key):
            del self._shares[key]
        if isinstance(alias, str):
            self._alias[key] = alias
        elif isclass(alias) or isfunction(alias):  # 注册类 & function
            self._alias[key] = alias
        elif isinstance(alias.__class__.__class__, type):  # 注册实例 (这边判断是否是实例 不太严谨)
            self._shares[key] = alias
            self._alias[key] = alias.__class__
        else:  # 其他
            self._alias[key] = alias
        return self

    def _instance(self, name):
        '''
        获取新的实例
        :param name: 对应的name
        :return: instance
        '''
        alias = self._alias.get(name)
        if alias is None:
            raise Exception('create instance fail, for {} not register'.format(name))
        if type(alias) == str:
            self._shares[name] = self._instance_by_str(alias)
        elif isfunction(alias):
            pass
        elif isinstance(alias, type):
            self._shares[name] = alias()
        elif isinstance(alias.__class__.__class__, type):
            self._shares[name] = alias.__class__()
        return self._shares.get(name) or self._alias[name]

    def _instance_by_str(self, alias):
        '''
        通过字符串实例华
        :param name: 注册调用使用的名称
        :param path: 路径
        :return:
        '''
        import importlib
        try:
            Import = importlib.import_module('{}'.format('.'.join(alias.split('.')[:-1])))
            classname = getattr(Import, alias.split('.')[-1])
            if isinstance(classname, type):
                return classname()
            elif isinstance(classname.__class__.__class__, type):
                return classname.__class__()
        except Exception as e:
            return alias

    def __call__(self, name):
        return self._shares.get(name) or self._instance(name)


app = App()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import requests


class Request():
    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(Request, cls).__new__(cls, *args, **kwargs)
            return cls._instance

    def get(self, url, params=None, header=None, timeout=5, **kwargs):
        return requests.get(url, params=params, headers=header, timeout=timeout, **kwargs)

    def post(self, url, params={}, timeout=5, header=None, **kwargs):
        return requests.post(url, data=params, headers=header, timeout=timeout, **kwargs)

    # 请求网络数据
    async def aiorequest(self, method, url, headers={}, params={}, data={},  **kwargs):
        import aiohttp
        async with aiohttp.ClientSession() as session:
            async with session.__getattribute__(method)(url, params=params, data=data, headers=headers, **kwargs) as response:
                res = await response.text()
                return Response(res)

    async def aioget(self, url, params={}, headers={}, **kwargs):
        return await self.aiorequest('get', url, params=params, headers=headers, **kwargs)

    async def aiopost(self, url, params={}, headers={}, **kwargs):
        return await self.aiorequest('post', url, data=params, headers=headers, **kwargs)


class Response():
    def __init__(self, text):
        self._text = text

    @property
    def text(self):
        return self._text

    def json(self):
        import json
        return json.loads(self._text)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""环境变量类"""

__author__ = ''

from configparser import ConfigParser
from pathlib import Path
from .App import App

class Env():
    """Env主要是为了获取.env中的配置数据"""

    _instance = None

    _config = None

    _config_parser = None

    basedir = None

    # Env为单例模式
    def __new__(cls, *args, **kw):
        if not cls._instance:
            cls._instance = super(Env, cls).__new__(cls, *args, **kw)
            cls._config_parser = MyConfigParser()
            cls._config_parser.read(Path(App()('BASE_PATH')).joinpath('.env'))
            cls._config = dict(cls._config_parser.items('environment'))
        return cls._instance

    def __call__(self, name, defaule=''):
        return self._config.get(name, defaule)

    def __getattr__(self, name):
        return self._config.get(name)

    def __getitem__(self, key):
        return self._config.get(key)

    def items(self, section, key=None, default=None):
        if key is None:
            return dict(self._config_parser.items(section))
        return dict(self._config_parser.items(section)).get(key, default)


class MyConfigParser(ConfigParser):
    """解决ConfigParser会将key名转成小写的问题"""

    def optionxform(self, str):
        return str


if __name__ == '__main__':
    import os
    App().register("BASE_PATH", os.path.dirname(os.path.dirname(__file__)))
    env = Env()
    print(env('LOG_PATH'))
    print(env('LOG_PATH1', "hahaha"))
    print(env.LOG_PATH)
    print(env['LOG_PATH'])
    print(env.items('sqlserver'))
    print(env.items('sqlserver', 'DB_HOST'))
    print(env.items('sqlserver', 'DB_HOST1', "123"))



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="simutil",
    version="1.0.2",
    author="jeanku, liubing",
    author_email="",
    description="A simple python libs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="",
    packages=["simutil", "simutil/Scaffold", "Config"],
    package_data={
        'simutil': [
            'Scaffold/*.txt'
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'configparser',
    ],
    keywords='util, libs',
    python_requires='>=3',
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import importlib


class ConfigParser(object):
    __loaded__ = dict()

    def __call__(self, module, default=None):
        module = module.split('.')
        key = '{}'.format('.'.join(module[:-1]))
        if self.__loaded__.get(key) is None:
            self.__loaded__[key] = importlib.import_module(key)
        return getattr(self.__loaded__[key], module[-1], default)

    def get(self, module, default=None):
        return self.__call__(module, default)


if __name__ == '__main__':
    data = ConfigParser()('Config.Filepath.RESOURCE_BASE_PATH')
    data = ConfigParser()('Config.Filepath.RESOURCE_BASE_PATH1', 'default')
    data = ConfigParser().get('Config.Filepath.RESOURCE_BASE_PATH')
    data = ConfigParser().get('Config.Settings.BLUR_MASK_MODIFIER', 'default')
    print(data)
